searchData={"items":[{"type":"module","title":"NervesTime","doc":"Keep time in sync on Nerves devices\n\n`NervesTime` keeps the system clock on [Nerves](http://nerves-project.org)\ndevices in sync when connected to the network and close to in sync when\ndisconnected. It's especially useful for devices lacking a [Battery-backed\nreal-time clock](https://en.wikipedia.org/wiki/Real-time_clock) and will\nadvance the clock at startup to a reasonable guess.\n\nNearly all configuration is via the application config (`config.exs`). The\nfollowing keys are available:\n\n* `:servers` - a list of NTP servers for time synchronization. Specifying an\n  empty list turns off NTP\n* `:time_file` - a file path for tracking the time. It allows the system to\n  start with a reasonable time quickly on boot and before the Internet is\n  available for NTP to work.\n* `:earliest_time` - times before this are considered invalid and adjusted\n* `:latest_time` - times after this are considered invalid and adjusted\n* `:ntpd` - the absolute path to the Busybox `ntpd`. This only needs to be\n  set if your system does not provide `ntpd` in the `$PATH`.\n* `:await_initialization_timeout` - Timeout to await a successful system time\n  initialization on startup before continuing asynchronously. Set in milliseconds.\n  Defaults to `0`.","ref":"NervesTime.html"},{"type":"function","title":"NervesTime.ntp_servers/0","doc":"Return the current NTP servers","ref":"NervesTime.html#ntp_servers/0"},{"type":"function","title":"NervesTime.restart_ntpd/0","doc":"Manually restart the NTP daemon\n\nThis is normally not necessary since `NervesTime` handles restarting it\nautomatically. An example of a reason to call this function is if you know\nwhen the Internet becomes available. For this case, calling `restart_ntp`\nwill cancel `ntpd`'s internal timeouts and cause it to immediately send time\nrequests. If using NTP Pool, be sure not to violate its terms of service by\ncalling this function too frequently.","ref":"NervesTime.html#restart_ntpd/0"},{"type":"function","title":"NervesTime.set_ntp_servers/1","doc":"Set the list of NTP servers\n\nUse this function to replace the list of NTP servers that are queried for\ntime. It is also possible to set this list in your `config.exs` by doing\nsomething like the following:\n\n```elixir\nconfig :nerves_time, :servers, [\n  \"0.pool.ntp.org\",\n  \"1.pool.ntp.org\",\n  \"2.pool.ntp.org\",\n  \"3.pool.ntp.org\"\n]\n```\n\n`NervesTime` uses [NTP Pool](https://www.ntppool.org/en/) by default. To\ndisable this and configure servers solely at runtime, specify an empty list\nin `config.exs`:\n\n```elixir\nconfig :nerves_time, :servers, []\n```","ref":"NervesTime.html#set_ntp_servers/1"},{"type":"function","title":"NervesTime.set_system_time/1","doc":"Set the system time","ref":"NervesTime.html#set_system_time/1"},{"type":"function","title":"NervesTime.synchronized?/0","doc":"Check whether NTP is synchronized with the configured NTP servers\n\nIt's possible that the time is already set correctly when this returns false.\n`NervesTime` decides that NTP is synchronized when `ntpd` sends a\nnotification that the device's clock stratum is 4 or less. Clock adjustments\noccur before this, though.\n\nOnce NTP is synchronized, it will remain that way until the `nerves_time`\napplication is restarted.","ref":"NervesTime.html#synchronized?/0"},{"type":"module","title":"NervesTime.FileTime","doc":"FileTime simulates a real-time clock using a file's mtime\n\nThe way it works is that a file is touched each time nerves_time\nwants to update the RTC. When nerves_time shuts down, the file is\nagain touched. The next boot then reads the last modified time of\nthe file so that nerves_time can set the clock to that value. It\nwill certainly be off, but not absurdly so unless the device has\nbeen powered off for a really long time.\n\nWhile this doesn't sound ideal at all, knowing the time within\nminutes, hours, or days can get the clock to within time ranges\nneeded for X.509 certificate validation.","ref":"NervesTime.FileTime.html"},{"type":"function","title":"NervesTime.FileTime.get_time/1","doc":"Return the timestamp of when update was last called or\nthe Unix epoch time (1970-01-01) should that not work.","ref":"NervesTime.FileTime.html#get_time/1"},{"type":"function","title":"NervesTime.FileTime.set_time/2","doc":"Update the file holding a stamp of the current time.","ref":"NervesTime.FileTime.html#set_time/2"},{"type":"function","title":"NervesTime.FileTime.terminate/1","doc":"Update the timestamp one final time","ref":"NervesTime.FileTime.html#terminate/1"},{"type":"function","title":"NervesTime.FileTime.time_file/0","doc":"Return the path to the file that keeps track of the time","ref":"NervesTime.FileTime.html#time_file/0"},{"type":"behaviour","title":"NervesTime.RealTimeClock","doc":"Behaviour for real-time clocks implementations.","ref":"NervesTime.RealTimeClock.html"},{"type":"callback","title":"NervesTime.RealTimeClock.get_time/1","doc":"Get the time from the clock\n\nThis is called after `init/1` returns successfully to see if the\nsystem clock should be updated.\n\nIf the time isn't set, the implementation should return `:unset`.\n`set_time/2` will be called when the time is known.","ref":"NervesTime.RealTimeClock.html#c:get_time/1"},{"type":"callback","title":"NervesTime.RealTimeClock.init/1","doc":"Initialize the clock\n\nThis is called when `nerves_time` starts. If it fails, `nerves_time`\nwon't call any of the other functions.","ref":"NervesTime.RealTimeClock.html#c:init/1"},{"type":"callback","title":"NervesTime.RealTimeClock.set_time/2","doc":"Set the clock\n\nThis is called if `nerves_time` determines that the implementation is out\nof sync with the true time and at regular intervals (usually 11 minutes) as\nupdates come in from NTP.\n\nIf the time can't be set, the implementation can either wait to be called\nthe next time or take some other action.","ref":"NervesTime.RealTimeClock.html#c:set_time/2"},{"type":"callback","title":"NervesTime.RealTimeClock.terminate/1","doc":"Clean up the clock state\n\nThis is called when `nerves_time` terminates. It's not guaranteed to be\ncalled, but if it is, it should clean up or do any final operations on\nthe RTC.","ref":"NervesTime.RealTimeClock.html#c:terminate/1"},{"type":"type","title":"NervesTime.RealTimeClock.state/0","doc":"Internal state of the hardware clock","ref":"NervesTime.RealTimeClock.html#t:state/0"},{"type":"module","title":"NervesTime.RealTimeClock.BCD","doc":"Convert between integers and binary-coded decimals (BCD)\n\nBCD is commonly used in Real-time clock chips for historical reasons. See\n[wikipedia.org/wiki/Binary-coded_decimal](https://en.wikipedia.org/wiki/Binary-coded_decimal)\nfor a good background on BCD. The BCD implementation here is referred to as\n\"Packed BCD\" in the article.","ref":"NervesTime.RealTimeClock.BCD.html"},{"type":"function","title":"NervesTime.RealTimeClock.BCD.from_integer/1","doc":"Convert a 8 bit integer value to a BCD binary","ref":"NervesTime.RealTimeClock.BCD.html#from_integer/1"},{"type":"function","title":"NervesTime.RealTimeClock.BCD.to_integer/1","doc":"Convert an 8 bit bcd-encoded value to an integer","ref":"NervesTime.RealTimeClock.BCD.html#to_integer/1"},{"type":"type","title":"NervesTime.RealTimeClock.BCD.t/0","doc":"Support two digit BCD","ref":"NervesTime.RealTimeClock.BCD.html#t:t/0"},{"type":"module","title":"NervesTime.Waiter","doc":"Waits for `NervesTime.SystemTime` to successfully set a sane real time.\n\nBy default that time is fetched completely async, but this waiter can be\nconfigured using `config :nerves_time, await_initialization_timeout: timeout` to\nblock startup for the configured duration. If the timeout elapses there's\nstill no guarantee for a sane real time being set. Setting the timeout\nto `:infinity` will however block until that happens.","ref":"NervesTime.Waiter.html"},{"type":"function","title":"NervesTime.Waiter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NervesTime.Waiter.html#child_spec/1"},{"type":"function","title":"NervesTime.Waiter.start_link/1","doc":"","ref":"NervesTime.Waiter.html#start_link/1"},{"type":"extras","title":"nerves_time","doc":"# nerves_time\n\n[![CircleCI](https://circleci.com/gh/nerves-time/nerves_time.svg?style=svg)](https://circleci.com/gh/nerves-time/nerves_time)\n[![Hex version](https://img.shields.io/hexpm/v/nerves_time.svg \"Hex version\")](https://hex.pm/packages/nerves_time)\n\n`NervesTime` keeps the system clock on [Nerves](http://nerves-project.org)\ndevices in sync when connected to the network and close to in sync when\ndisconnected. It's especially useful for devices lacking a [Battery-backed\nreal-time clock](https://en.wikipedia.org/wiki/Real-time_clock) and will advance\nthe clock at startup to a reasonable guess.","ref":"readme.html"},{"type":"extras","title":"Installation - nerves_time","doc":"First add `nerves_time` to your project's dependencies:\n\n```elixir\ndef deps do\n  [\n    {:nerves_time, \"~> 0.4.2\"}\n  ]\nend\n```\n\nEnsure that your `vm.args` allows for\n[timewarps](http://erlang.org/doc/apps/erts/time_correction.html#time-warp-modes).\nIf it doesn't, `nerves_time` will update the OS system time, but Erlang's system\ntime will lag. The following line should be in the beginning or middle of the\n`vm.args` file:\n\n```elixir\n+C multi_time_warp\n```\n\nIf you're using one of the official Nerves Systems, then this is all that's\nneeded. `nerves_time` requires Busybox's `ntpd` and `date` applets to be\nenabled. If you haven't explicitly disabled them, they're probably enabled.","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - nerves_time","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"Startup - nerves_time","doc":"`nerves_time` by default does not block waiting for a valid system time to be set.\nThis can result in your application running before the time has been adjusted, which\nmay be undesirable. To lessen the likelyhood of that happening you can adjust\nthe `:await_initialization_timeout` config to wait for a valid system time to be set.\nIf `nerves_time` fails to do that within the given timeframe it will stop blocking\nstartup and continue trying asynchronously.\n\n```elixir\n# config/config.exs\n\nconfig :nerves_time, await_initialization_timeout: :timer.seconds(5)\n```","ref":"readme.html#startup"},{"type":"extras","title":"NTP - nerves_time","doc":"`nerves_time` uses [ntp.pool.org](https://www.ntppool.org/en/) for time\nsynchronization. Please see their [terms of\nuse](https://www.ntppool.org/tos.html) before tweaking `nerves_time`.\nAlternative NTP servers can be specified using the `config.exs`:\n\n```elixir\n# config/config.exs\n\nconfig :nerves_time, :servers, [\n    \"0.pool.ntp.org\",\n    \"1.pool.ntp.org\",\n    \"2.pool.ntp.org\",\n    \"3.pool.ntp.org\"\n  ]\n```\n\nIt's also possible to configure NTP servers at runtime. See\n`NervesTime.set_ntp_servers/1`.","ref":"readme.html#ntp"},{"type":"extras","title":"Valid time range - nerves_time","doc":"`nerves_time` has a concept of a valid time range. This minimizes time\nerrors on systems without clocks or Internet connections or that may have some\nissue that causes a very wrong time value. The default valid time range is\nhardcoded and moves forward each release. It is not the build timestamp since\nthat results in [non-reproducible builds](https://reproducible-builds.org).\nApplications can override the valid range via the application config:\n\n```elixir\n# config/config.exs\n\nconfig :nerves_time, earliest_time: ~N[2019-10-04 00:00:00], latest_time: ~N[2022-01-01 00:00:00]\n```","ref":"readme.html#valid-time-range"},{"type":"extras","title":"Algorithm - nerves_time","doc":"Here's the basic idea behind `nerves_time`:\n\n* If the clock hasn't been set or is invalid, set it to the earliest valid\n  time known to `nerves_time`. This is either set in the application config or\n  defaulted to a reasonable value that likely moves forward a little each\n  `nerves_time` release.\n* check for time via a [Real Time Clock](#Real-Time-Clock)\n* Run Busybox `ntpd` to synchronize time using the [NTP\n  protocol](https://en.wikipedia.org/wiki/Network_Time_Protocol).\n* Update [Real Time Clock](#Real-Time-Clock) periodically and on graceful power\n  downs. This is currently only done at around 11 minute intervals.\n\nTo check the NTP synchronization status, call `NervesTime.synchronized?/0`.","ref":"readme.html#algorithm"},{"type":"extras","title":"Real Time Clock - nerves_time","doc":"A hardware based real time clock can be configured by added a config.exs entry:\n\n```elixir\nconfig :nerves_time, rtc: {SomeImplementingModule, [some: :initialization_opt]}\n```\n\nBy default Nerves Time is configured to use `NervesTime.FileTime` which will\nCheck for `~/.nerves_time`. If it exists, advance the clock to it's last\nmodification time.\n\nSee the documentation for `NervesTime.RealTimeClock` to implement your own\nreal time clock.","ref":"readme.html#real-time-clock"},{"type":"extras","title":"Credits and license - nerves_time","doc":"This project started as a fork of\n[nerves_ntp](https://hex.pm/packages/nerves_ntp) by Marcin Operacz and Wojciech\nMandrysz. It has quite a few changes from since when they worked on the project,\nbut some of their code still exists. Both their project and this one are covered\nby the [Apache-2.0 license](https://opensource.org/licenses/Apache-2.0).","ref":"readme.html#credits-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.4.8 - Changelog","doc":"* Fixes\n  * Fix \"Cannot initialize rtc\" error that's in v0.4.7. (thanks @joshk)\n  * Add \"[NervesTime]\" to log messages to make it easier to identify where they\n    come from.","ref":"changelog.html#v0-4-8"},{"type":"extras","title":"v0.4.7 - Changelog","doc":"* Updates\n  * Don't create communications socket file if not starting `ntpd` so that it\n    doesn't need to be cleaned up. (thanks @joshk)\n  * Add and clean up missing typespecs","ref":"changelog.html#v0-4-7"},{"type":"extras","title":"v0.4.6 - Changelog","doc":"* Updates\n  * Build cleanly on Elixir 1.15. This raises the minimum supported Elixir\n    version to 1.11 since we no longer test on early versions.","ref":"changelog.html#v0-4-6"},{"type":"extras","title":"v0.4.5 - Changelog","doc":"* New feature\n  * Support blocking NervesTime startup until initialized with a valid time.\n    See [Startup](https://github.com/nerves-time/nerves_time#startup) doc and\n    `NervesTime.Waiter` for more info (thanks @LostKobrakai :heart:)","ref":"changelog.html#v0-4-5"},{"type":"extras","title":"v0.4.4 - Changelog","doc":"* Updates\n  * Allow `muontrap v1.0.0` to be used.","ref":"changelog.html#v0-4-4"},{"type":"extras","title":"v0.4.3 - Changelog","doc":"This release only reduces Makefile prints and has no code changes. It is a safe\nupdate.","ref":"changelog.html#v0-4-3"},{"type":"extras","title":"v0.4.2 - Changelog","doc":"* New feature\n  * Added `NervesTime.set_system_time/1` to manually set the system clock.\n    Thanks to Eric Rauer for this.","ref":"changelog.html#v0-4-2"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"* Bug fixes\n  * Fix crash when setting time.","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"* New features\n  * Added `NervesTime.RealTimeClock` behaviour to support external RTC\n    implementations. The default RTC is still FileTime. I.e., approximate an RTC\n    by periodically updating the modified time on a file and reading it back at\n    boot. See [github.com/nerves-time](https://github.com/nerves-time) for\n    example RTC implementations.\n\n* Bug fixes\n  * `NervesTime.synchronized?/0` will stay true once NTP synchronizes. It will\n    not revert to false unless a crash happens and the NTP code restarts. This\n    should address some confusion, since in practice the function was being used\n    to check whether the system time could be used. Bouncing the answer between\n    true and false depending on ntp reports was confusing since the system time\n    had already been set once and was plenty close to the real time.","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.2 - Changelog","doc":"* Improvements\n  * Further reduce ntpd's logging. It is just too much especially when the\n    internet is down.","ref":"changelog.html#v0-3-2"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"* Bug fixes\n  * Move ntpd's prints to the log. The previous prints to the console were quite\n    annoying to say the least.\n  * Remove the build timestamp so that `nerves_time` builds are reproducible.\n    This is important for users that want to recreate firmware images with as\n    few differences as possible. The build timestamp had restricted the earliest\n    allowed time. The earliest time is now hardcoded, but can be overridden by\n    application config (same with the latest possible time.)","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"IMPORTANT: This release moves `Nerves.Time` to the `NervesTime` namespace. If\nyou are using the API, you will need to rename every instance of `Nerves.Time`\nin your code to `NervesTime`.\n\n* Bug fixes\n  * `ntpd` crash detection and restart have been simplified by using\n    `MuonTrap.Daemon`. This should fix an issue that was seen with `ntpd` not\n    getting restarted after it crashed.\n  * `NervesTime.synchronized?/0` would return synchronized when changing NTP\n    servers and right after restarting `ntpd`. It will return `false` now and\n    switch to `true` when time really has been synchronized.\n\n* Improvements\n  * Simplified ntpd reporting code. No more regular expressions. Reports are\n    sent via a Unix Domain socket and encoded as Erlang terms. This deleted a\n    lot of string parsing code that felt brittle.\n  * Added a restart delay on unclean `ntpd` restarts to prevent pegging public\n    NTP servers. The delay is currently a minute since `ntpd` crashes are not\n    well-understood. Luckily, this seems like a rare event.\n  * Added more tests to cover NTP use","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.1 - Changelog","doc":"* Improvements\n  * Move C build products under `_build`. This lets you switch targets with out\n    cleaning builds in between.","ref":"changelog.html#v0-2-1"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"* Added support for configuring NTP servers at runtime. Thanks to @ConnorRigby\n  for sending a PR for this\n* Added `Nerves.Time.restart_ntpd/0` for users who know external information\n  about good times to bounce the NTP daemon so that it syncs more quickly\n* Fixed naming of `is_synchronized/0` to be `synchronized?/0`. Thanks to\n  @brodeuralexis for catching my slip.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}